package Cpanel::CryptoPhoto;

## Copyright (c) 2014 CryptoPhoto -- http://cryptophoto.com
##
## AUTHORS:
##   CryptoPhoto
##
## Permission is hereby granted, free of charge, to any person obtaining a copy
## of this software and associated documentation files (the "Software"), to deal
## in the Software without restriction, including without limitation the rights
## to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
## copies of the Software, and to permit persons to whom the Software is
## furnished to do so, subject to the following conditions:
##
## The above copyright notice and this permission notice shall be included in
## all copies or substantial portions of the Software.
##
## THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
## IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
## FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
## LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
## OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
## THE SOFTWARE.

#                                                  Copyright(c) 2014 CryptoPhoto
#                                                           All rights Reserved.
# copyright@cryptophoto.com                               http://cryptophoto.com

use strict;
use Cpanel::SafeFile             ();
use Cpanel::SafeDir              ();
use Cpanel::FileUtils::TouchFile ();
use Cpanel::Logger               ();
use Cpanel::Config               ();
use Cpanel::Locale               ();
use Digest::MD5                qw(md5_hex);
use Cpanel::CryptoPhoto::Utils;

our $VERSION = '1.0';

my $logger = Cpanel::Logger->new();
my $CPTOOL = '/usr/local/cpanel/base/3rdparty/cryptophoto/cptool';
my $CP_CONF_FILE = '/usr/local/cpanel/base/3rdparty/cryptophoto/cryptophoto.conf';
my $users_dir = '/usr/local/cpanel/base/3rdparty/cryptophoto/users/'; 


sub CryptoPhoto_init {
  my $settings_file = $users_dir . $Cpanel::user;

  if(!-e $settings_file) {

    Cpanel::FileUtils::TouchFile::touchfile($settings_file);
    if(-e $settings_file) {
      my $conf = { 'enabled' => 0 };
      Cpanel::CryptoPhoto::Utils::flushConfig($conf, $settings_file);
      chmod 0600, $settings_file;
    }
  }


  return 1;
}

sub CryptoPhoto_active {
  return _active();
}

sub CryptoPhoto_switch {
    #if ( !main::hasfeature("ipdeny") ) { return; }

    my $active = _active();

    if($active) {
      _active(0);
      $active = 0;
    } else {
      _active(1);
      $active = 1;
    }

    return;
}

sub CryptoPhoto_show_form {

  my $locale = Cpanel::Locale->get_handle();

  my $active = $locale->maketext('CryptoPhoto_inactive') . "  <input type='submit' name='switch' value='" . $locale->maketext('CryptoPhoto_activate') . "'>";

  if(_active()) {
    $active = $locale->maketext('CryptoPhoto_active') . " <input type='submit' name='switch' value='" . $locale->maketext('CryptoPhoto_deactivate') . "'>";
  }

  my $extra = "";

  if (-e $CP_CONF_FILE) {
    my $config = Cpanel::CryptoPhoto::Utils::load_Config($CP_CONF_FILE);

    if (!$config->{'server'}) {
      $config->{'server'} = "https://cryptophoto.com";
    }

    if ($config->{'privateapikey'} && $config->{'publicapikey'} && $config->{'salt'} && $config->{'policy'}) {

      my $cmd = $CPTOOL . ' -privateapikey="' . $config->{'privateapikey'} . '" -publicapikey="' . $config->{'publicapikey'} . '" -uid="' .
          md5_hex($config->{'salt'} . $Cpanel::user) . '" -server="' . $config->{'server'} . '" -command="get_gen_widget" -ip="'.$ENV{'REMOTE_ADDR'}.'"';

      $cmd =~ /^(.)(.*)$/; $cmd = $1.$2;

      my $out = `$cmd`;

      if($out !~ /failure/i) {
        $extra = $out;
      }

    } else {
      if ($config->{'privateapikey'} && $config->{'publicapikey'} && $config->{'salt'}) {
        print $locale->maketext('CryptoPhoto_is_not_active');
      } else {
        print $locale->maketext('CryptoPhoto_is_not_configured');
      }
      return;
    }
  }

  my $service_txt = $locale->maketext('CryptoPhoto_service');

  my $form =<<EOF;
<form method="POST">
$service_txt $active
<div id="cploading"style="margin-top:10px"><center><img src="img/loading.gif" style="display: inline; width: 16px; height: 16px;" border="0"> Loading CryptoPhoto Panel...</center></div>
<script type="text/javascript">var hide="cploading";</script>
<div id="cp_widget" class="cp_widget" style="margin-top:10px;margin-left:auto;margin-right:auto"></div>
</form>
$extra
EOF
  print $form;
  return;
}

sub _active {
  my ($value) = @_;

  my $settings_file = $users_dir . $Cpanel::user;

  if (-e $settings_file ) {
    my $conf = Cpanel::CryptoPhoto::Utils::load_Config($settings_file);

    if (defined $value) {
      $conf->{'enabled'} = $value;
      Cpanel::CryptoPhoto::Utils::flushConfig($conf, $settings_file);
    }
    return $conf->{'enabled'};
  }

  return 0;
}

1;
