package Cpanel::Security::Policy::CryptoPhoto;

## Copyright (c) 2014 CryptoPhoto -- http://cryptophoto.com
##
## AUTHORS:
##   CryptoPhoto
##
## Permission is hereby granted, free of charge, to any person obtaining a copy
## of this software and associated documentation files (the "Software"), to deal
## in the Software without restriction, including without limitation the rights
## to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
## copies of the Software, and to permit persons to whom the Software is
## furnished to do so, subject to the following conditions:
##
## The above copyright notice and this permission notice shall be included in
## all copies or substantial portions of the Software.
##
## THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
## IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
## FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
## LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
## OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
## THE SOFTWARE.

#                                                  Copyright(c) 2014 CryptoPhoto
#                                                           All rights Reserved.
# copyright@cryptophoto.com                               http://cryptophoto.com

use base 'Cpanel::SecurityPolicy::Base';

use Digest::MD5                qw(md5_hex);
use Cpanel::SafeDir              ();
use Cpanel::Logger               ();
use Cpanel::Config               ();
use Cpanel::CryptoPhoto::Utils;

my $CPTOOL = '/usr/local/cpanel/base/3rdparty/cryptophoto/cptool';
my $CP_CONF_FILE = '/usr/local/cpanel/base/3rdparty/cryptophoto/cryptophoto.conf';
my $users_dir = '/usr/local/cpanel/base/3rdparty/cryptophoto/users/';

my $logger = Cpanel::Logger->new();

sub new {
    my ($class) = @_;

    # Compiler does not necessarily properly load the base class.
    unless ( exists $INC{'Cpanel/SecurityPolicy/Base.pm'} ) {
        eval 'require Cpanel::SecurityPolicy::Base;';
    }
    return Cpanel::SecurityPolicy::Base->init( __PACKAGE__, 500 );
}

sub fails {
    my ( $self, $sec_ctxt, $cpconf_ref ) = @_;

    my $cookie_ref = $sec_ctxt->{'cookies'};

    if ($sec_ctxt->{"request_type"} ne "normal" || $sec_ctxt->{'appname'} ne "cpaneld") {
        return 0;
    }
    if ( !$sec_ctxt->{'is_possessed'} && $sec_ctxt->{'virtualuser'} ) {
        return 0;
    }
    if ( $sec_ctxt->{'auth_by_accesshash'} ) {
        return 0;
    }

    my $user;
    if ( $sec_ctxt->{'is_possessed'} ) {
        $user = $sec_ctxt->{'possessor'};
    }
    else {
        $user = $sec_ctxt->{'user'};
    }
    $user =~ /(.*)/;    # TODO: brute-force untaint
    $user = $1;

    my $settings_file = $users_dir . $user;

    my $user_conf = Cpanel::CryptoPhoto::Utils::load_Config($settings_file);
    my $cp_config = Cpanel::CryptoPhoto::Utils::load_Config($CP_CONF_FILE);

    my $cpsession = defined $cookie_ref->{'cpsession'} ? md5_hex($cookie_ref->{'cpsession'}) : defined $user_conf->{'cpsession'} ? $user_conf->{'cpsession'} : "";

    if (defined $user_conf->{'skip'} && $user_conf->{'skip'} eq $cpsession ) {
       return 0;
    }

    if (!$cp_config->{'server'}) {
      $cp_config->{'server'} = "https://cryptophoto.com";
    }

    if ($cp_config->{'privateapikey'} && $cp_config->{'publicapikey'} && $cp_config->{'salt'} && $user_conf->{'enabled'}) {

       my $cmd = $CPTOOL . ' -privateapikey="' . $cp_config->{'privateapikey'} . '" -publicapikey="' . $cp_config->{'publicapikey'} . '" -uid="' .
                 md5_hex($cp_config->{'salt'} . $user) . '" -server="' . $cp_config->{'server'} . '" -command="get_session" -ip="'.$ENV{'REMOTE_ADDR'}.'"';

       $cmd =~ /^(.)(.*)$/;
       $cmd = $1.$2;

       my $out = `$cmd`;

       if($out !~ /failure/i) {
          my($sd,$hastoken) = split(/\|/, $out);
          if ($cpsession) {
            $user_conf->{"cpsession"} = $cpsession;
            Cpanel::CryptoPhoto::Utils::flushConfig($user_conf, $settings_file);
          }
          return 1 if($hastoken);
       }
    }
    return 0;
}

1;

