package Cpanel::Security::Policy::CryptoPhoto::UI::HTML;

## Copyright (c) 2014 CryptoPhoto -- http://cryptophoto.com
##
## AUTHORS:
##   CryptoPhoto
##
## Permission is hereby granted, free of charge, to any person obtaining a copy
## of this software and associated documentation files (the "Software"), to deal
## in the Software without restriction, including without limitation the rights
## to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
## copies of the Software, and to permit persons to whom the Software is
## furnished to do so, subject to the following conditions:
##
## The above copyright notice and this permission notice shall be included in
## all copies or substantial portions of the Software.
##
## THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
## IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
## FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
## LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
## OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
## THE SOFTWARE.

#                                                  Copyright(c) 2014 CryptoPhoto
#                                                           All rights Reserved.
# copyright@cryptophoto.com                               http://cryptophoto.com


use Digest::MD5                       qw(md5_hex);
use Cpanel::Locale                      ();
use Cpanel::Logger                      ();
use Cpanel::Encoder::Tiny;
use Cpanel::CryptoPhoto::Utils;

my $locale;

my $logger = Cpanel::Logger->new();
my $CPTOOL = '/usr/local/cpanel/base/3rdparty/cryptophoto/cptool';
my $CP_CONF_FILE = '/usr/local/cpanel/base/3rdparty/cryptophoto/cryptophoto.conf';
my $users_dir = '/usr/local/cpanel/base/3rdparty/cryptophoto/users/';

sub new {
    my ( $class, $policy ) = @_;
    die "No policy object supplied.\n" unless defined $policy;
    return bless { 'policy' => $policy }, $class;
}

sub process {
    my ( $self, $formref, $sec_ctxt, $cpconf_ref ) = @_;

    my $cookie_ref = $sec_ctxt->{'cookies'};

    my $user;
    if ( $sec_ctxt->{'is_possessed'} ) {
        $user = $sec_ctxt->{'possessor'};
    }
    else {
        $user = $sec_ctxt->{'user'};
    }

    $user =~ /(.*)/;    # TODO: brute-force untaint
    $user = $1;

    my $settings_file = $users_dir . $user;

    my $user_conf = Cpanel::CryptoPhoto::Utils::load_Config($settings_file);

    if (!$user_conf->{'enabled'}) {
       _redirect_home($sec_ctxt);
       return;
    }

    my $cp_config = Cpanel::CryptoPhoto::Utils::load_Config($CP_CONF_FILE);

    if (!$cp_config->{'server'}) {
      $cp_config->{'server'} = "https://cryptophoto.com";
    }

    if (!$cp_config->{'publicapikey'} || !$cp_config->{'privateapikey'} || !$cp_config->{'salt'} || !$cp_config->{'policy'}) {
      _redirect_home($sec_ctxt);
      return;
    }

    $locale ||= Cpanel::Locale->get_handle();
    my $error = "";

    if ( (defined $formref->{'cp_auth'} && $formref->{'cp_auth'}) || (defined $formref->{'cp_phc'} || $formref->{'cp_phc'}) ) {
      my ($selector) = $formref->{'token_selector'} ? $formref->{'token_selector'} : ""; 
      my ($row)      = $formref->{'token_response_field_row'} ? $formref->{'token_response_field_row'} : "";
      my ($col)      = $formref->{'token_response_field_col'} ? $formref->{'token_response_field_col'} : "";
      my ($cp_phc)   = $formref->{'cp_phc'} ? $formref->{'cp_phc'} : "";
      my ($ip)       = $ENV{'REMOTE_ADDR'};
       
      if ($cp_config->{'privateapikey'} && $cp_config->{'publicapikey'} && $cp_config->{'salt'}) {
        my $parms = "${selector}:::${row}:::${col}:::${cp_phc}:::${ip}";
        my $cmd = $CPTOOL . ' -privateapikey="' . $cp_config->{'privateapikey'} . '" -publicapikey="' . $cp_config->{'publicapikey'} . '" -uid="' .
                  md5_hex($cp_config->{'salt'} . $user) . '" -server="' . $cp_config->{'server'} . '" -command="verify" -parms="'.$parms.'"';

        $cmd =~ /^(.)(.*)$/; $cmd = $1.$2;
        my $out = `$cmd`;

        if($out =~ /success/i) {

          my $cpsession = defined $cookie_ref->{'cpsession'} ? md5_hex($cookie_ref->{'cpsession'}) : defined $user_conf->{'cpsession'} ? $user_conf->{'cpsession'} : "";

          if ($cpsession) {
            $user_conf->{'skip'} = $cpsession;
            Cpanel::CryptoPhoto::Utils::flushConfig($user_conf, $settings_file);
          }

          _redirect_home($sec_ctxt);

          return;
        } else {
          if ($out !~ /incorrect-retry-count/i) {
            $error = $locale->maketext('CryptoPhoto_auth_error');
          }
        }
      }
    } 

    my %template_vars = (
        'cp_security_token' => $ENV{'cp_security_token'}, 
        'auth_widget'       => _get_auth_form($user, $cp_config),
        'cp_error'          => $error,
    );

    process_appropriate_template(
        'app'  => $sec_ctxt->{'appname'},
        'file' => 'cp',
        'data' => \%template_vars,
    );

    return;
}

sub process_appropriate_template {
    my (%opts) = @_;
    Cpanel::SecurityPolicy::UI::html_header();
    Cpanel::SecurityPolicy::UI::process_template( "CryptoPhoto/$opts{'file'}.html.tmpl", $opts{'data'} );
    Cpanel::SecurityPolicy::UI::html_footer();
}

sub _get_auth_form {
  my ($user, $config) = @_;

  my $extra = "";

  if ($config->{'privateapikey'} && $config->{'publicapikey'} && $config->{'salt'}) {

    my $cmd = $CPTOOL . ' -privateapikey="' . $config->{'privateapikey'} . '" -publicapikey="' . $config->{'publicapikey'} . '" -uid="' .
              md5_hex($config->{'salt'} . $user) . '" -server="' . $config->{'server'} . '" -command="get_auth_widget" -ip="'.$ENV{'REMOTE_ADDR'}.'"';

    $cmd =~ /^(.)(.*)$/; $cmd = $1.$2;
    my $out = `$cmd`;

    if($out !~ /failure/i) {
      $extra = $out;
    }
  }
  return $extra;
}

sub _redirect_home {
  my ($acctref) = @_;

  my $theme = 'x3';
  $theme = $acctref->{'cptheme'} if defined $acctref->{'cptheme'} && -f "/usr/local/cpanel/base/frontend/$acctref->{'cptheme'}/index.html";
  Cpanel::SecurityPolicy::UI::force_redirect("$ENV{'cp_security_token'}/frontend/$theme/index.html");

}

1;
