#!/usr/bin/perl
#WHMADDON:cryptophoto:CryptoPhoto

## Copyright (c) 2014 CryptoPhoto -- http://cryptophoto.com
##
## AUTHORS:
##   CryptoPhoto
##
## Permission is hereby granted, free of charge, to any person obtaining a copy
## of this software and associated documentation files (the "Software"), to deal
## in the Software without restriction, including without limitation the rights
## to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
## copies of the Software, and to permit persons to whom the Software is
## furnished to do so, subject to the following conditions:
##
## The above copyright notice and this permission notice shall be included in
## all copies or substantial portions of the Software.
##
## THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
## IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
## FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
## AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
## LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
## OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
## THE SOFTWARE.

#                                                  Copyright(c) 2014 CryptoPhoto
#                                                           All rights Reserved.
# copyright@cryptophoto.com                               http://cryptophoto.com


BEGIN { unshift @INC, '/usr/local/cpanel' }

use strict;
use warnings;
use Cpanel::Logger                  ();
use Cpanel::Config                  ();
use Cpanel::Form                    ();
use Cpanel::SafeFile                ();
use Digest::MD5            qw(md5_hex);
use Whostmgr::ACLS                  ();
use Cpanel::CryptoPhoto::Utils;

my $logger = Cpanel::Logger->new();
my $CPTOOL = '/usr/local/cpanel/base/3rdparty/cryptophoto/cptool';
my $CPTOOL_TEST = '/usr/local/cpanel/base/3rdparty/cryptophoto/cptool-test';
my $CP_CONF_FILE = '/usr/local/cpanel/base/3rdparty/cryptophoto/cryptophoto.conf';


my $cp_config = Cpanel::CryptoPhoto::Utils::load_Config($CP_CONF_FILE);
my $user = 0;


# Only root access
Whostmgr::ACLS::init_acls();
if ( !Whostmgr::ACLS::hasroot() ) {
    print "Content-type: text/html\r\n\r\n";
    print qq{
<br />
<br />
<div align="center"><h1>Permission denied</h1></div>
</body>
</html>
    };
    exit;
}

my %FORM = Cpanel::Form::parseform();

my $cmd = "index";

if($FORM{'cmd'}) {
  $cmd = lc($FORM{'cmd'});
}

my $security_token = "";
$security_token = $ENV{'cp_security_token'} if(defined $ENV{'cp_security_token'});

Cpanel::CryptoPhoto::Utils::dumper($logger, \%FORM, "FORM");
Cpanel::CryptoPhoto::Utils::dumper($logger, \%ENV, "ENV");

if ($cmd eq "settings") {
  do_settings(\%FORM, $security_token);
} 
elsif ($cmd eq "users") {
  do_users(\%FORM, $security_token);
}
elsif ($cmd eq "users_list") {
  do_users_list(\%FORM, $security_token);
}
elsif ($cmd eq "user_switch") {
  do_user_switch(\%FORM, $security_token);
} else {
  do_index(\%FORM, $security_token);
}

exit;

sub do_settings {
  my($FORM, $sc) = @_;

  my $header = header("settings", $sc);

  my $CP_CONF_FILE = '/usr/local/cpanel/base/3rdparty/cryptophoto/cryptophoto.conf';
  my $cryptophoto_conf_ref = Cpanel::CryptoPhoto::Utils::load_Config($CP_CONF_FILE);
  my $users_dir = '/usr/local/cpanel/base/3rdparty/cryptophoto/users/';

  $cryptophoto_conf_ref->{'salt'} = "" unless($cryptophoto_conf_ref->{'salt'});
  $cryptophoto_conf_ref->{'privateapikey'} = "" unless($cryptophoto_conf_ref->{'privateapikey'});
  $cryptophoto_conf_ref->{'publicapikey'} = "" unless($cryptophoto_conf_ref->{'publicapikey'});
  $cryptophoto_conf_ref->{'server'} = "" unless($cryptophoto_conf_ref->{'server'});

  my $save = "";
  my $test_config = "";
  my $result = "";

  if($FORM{'save'}) {
    my $salt          = $FORM{'salt'} ? $FORM{'salt'} : "";
    my $publicapikey  = $FORM{'publicapikey'}  ? $FORM{'publicapikey'} : "";
    my $privateapikey = $FORM{'privateapikey'} ? $FORM{'privateapikey'} : "";
    my $server = $FORM{'server'} ? $FORM{'server'} : "https://cryptophoto.com/api";

    my $update = 0;

    if ($salt) {
      $cryptophoto_conf_ref->{'salt'} = $salt;
      $update = 1;
    } else {
      $cryptophoto_conf_ref->{'salt'} = "";
    }

    if ($privateapikey) {
      $cryptophoto_conf_ref->{'privateapikey'} = $privateapikey;
      $update = 1;
    } else {
      $cryptophoto_conf_ref->{'privateapikey'} = "";
    }

    if ($publicapikey) {
      $cryptophoto_conf_ref->{'publicapikey'} = $publicapikey;
      $update = 1;
    } else {
      $cryptophoto_conf_ref->{'publicapikey'} = "";
    }

    if ($server) {
      $cryptophoto_conf_ref->{'server'} = $server;
      $update = 1;
    } else {
      $cryptophoto_conf_ref->{'server'} = "https://cryptophoto.com";
    }

    if ($update) {
      Cpanel::CryptoPhoto::Utils::flushConfig( $cryptophoto_conf_ref, $CP_CONF_FILE );
      $save = '<div id="alert_success"><div id="alert_img"><img src="cryptophoto/images/success.gif" alt="Success Icon" border="0"></div>
               <div id="alert_content"><span class="message">Changes stored successfully</span></div></div>';
    }
  } elsif ($FORM{'test'}) {
      # test CP api

      if (!$cp_config->{'server'}) {
	$cp_config->{'server'} = "https://cryptophoto.com";
      }

      my $message = ""; 
      if ($cp_config->{'privateapikey'} && $cp_config->{'publicapikey'} && $cp_config->{'salt'}) {
        my $user = 0;
        my $cmd = $CPTOOL . ' -privateapikey="' . $cp_config->{'privateapikey'} . '" -publicapikey="' . $cp_config->{'publicapikey'} . '" -uid="' .
                  md5_hex($cp_config->{'salt'} . $user) . '" -server="' . $cp_config->{'server'} . '" -command="get_session" -ip="'.$ENV{'REMOTE_ADDR'}.'"';

        $cmd =~ /^(.)(.*)$/;
        $cmd = $1.$2;

        my $out = `$cmd`;
        if($out =~ /failure/i) {

          $out =~ m/failure(.*)/i; 
   
          $out = $1;
          chomp($out);
          if ($out =~ m/inexistent\-provider/i) {
            $message = "Error: Please verify that the API keys are correct";
          } 
          elsif ($out =~ m/incorrect\-timestamp/i) {
            $message = "Error: Please verify that the date/time is properly set on this server";
          } 
          elsif ($out =~ m/(ip\-not\-allowed\s\|\s)(.*)/i) {
            $message = "Error: The IP of this server(" . $2 . ") is not added to the list of allowed IPs. Check your settings in your CryptoPhoto.com account";
          } 
          elsif ($out =~ m/incorrect\-signature/i || $out =~ m/incorrect\-api\-request/i) {
            $message = "Error: Please verify that the API keys are valid";
          } 
          else {
            $message = "Error: Unknown error $out<br/><br/>";
            my $test = `perl $CPTOOL_TEST`;
            $message .= "<pre>$test</pre>";
          }

          $test_config = '<div id="alert_error" style="background-color: #F6F6F6; border: 1px solid #CCC;width: 100%;">
                          <div id="alert_img"><img src="cryptophoto/images/error.gif" alt="Error Icon" border="0"></div>
                          <div id="alert_content"><span class="message" style="color:red;">' . $message . '</span></div></div>';
        }
        else {
	  if ($out !~ /[a-z0-9]\|(0|1)/i) {
            $message = `perl $CPTOOL_TEST`;

            $message .= $out;

            $test_config = '<div id="alert_error" style="background-color: #F6F6F6; border: 1px solid #CCC;width: 100%;">
                            <div id="alert_img"><img src="cryptophoto/images/error.gif" alt="Error Icon" border="0"></div>
                            <div id="alert_content"><span class="message" style="color:red;">Error: Unknown error <br/><br/><pre>' . $message . '</pre></span></div></div>';

          } else {

            # successfuly configured
            $message = "CryptoPhoto plugin is properly configured";
            $save = '<div id="alert_success"><div id="alert_img"><img src="cryptophoto/images/success.gif" alt="Success Icon" border="0"></div>
                            <div id="alert_content"><span class="message">' . $message . '</span></div></div>';

          }

         }
       }
       else {
         $message = "Please verify that your API keys are set and valid";
         $test_config = '<div id="alert_error" style="background-color: #F6F6F6; border: 1px solid #CCC;">
                         <div id="alert_img"><img src="cryptophoto/images/error.gif" alt="Error Icon" border="0"></div>
                         <div id="alert_content"><span class="message" style="color:red;">' . $message . '</span></div></div>';
       }
    }

my $HTML =<<HTML;
<!DOCTYPE html>
<html>
  <head>
  <link href="cryptophoto/css/style.css" rel="stylesheet" media="all" type="text/css" />
  </head>
  <body>
  $header
  <div class="container">
    <form method="POST">
      <p>
         CryptoPhoto API Keys can be obtain when registering a new account with CryptoPhoto (you can register a new CryptoPhoto
         account <a href="https://cryptophoto.com/admin/register" target="_blank">here</a>). <br/>
         More details and information about this plugin can be found <a href="http://cryptophoto.com/info/cpanel" target="_blank">here</a>.
      </p>
      $save

      <input type="hidden" name="cmd" value="settings">
      <div class="legend">Salt for generating users ID:</div><input type="text" name="salt" value="$cryptophoto_conf_ref->{'salt'}" style="width:350px"><br/>
      <div class="legend">CryptoPhoto Public ApiKey:</div><input type="text" name="publicapikey" value="$cryptophoto_conf_ref->{'publicapikey'}" style="width:350px"><br/>
      <div class="legend">CryptoPhoto Private ApiKey:</div><input type="text" name="privateapikey" value="$cryptophoto_conf_ref->{'privateapikey'}" style="width:350px"><br/>
      <div class="legend">CryptoPhoto Server:</div><input type="text" name="server" value="$cryptophoto_conf_ref->{'server'}" style="width:350px"><br/>
      <br/>
      <input type="submit" name="save" value="Save">
      <input type="submit" name="test" value="Test Configuration">
      <div>$result</div>
      <br/>
      $test_config
    </form>
  </div>
  </body>
  </html>
HTML

  print "Content-Type: text/html\n\n";
  print $HTML;
}

sub do_users {
  my($FORM, $sc) = @_;

  my $header = header("users", $sc);
  
  my $HTML =<<HTML;
<!DOCTYPE html>
<html>
  <head>
  <link href="cryptophoto/css/style.css" rel="stylesheet" media="all" type="text/css" />
  <script src="cryptophoto/js/jquery.min.js" type="text/javascript"></script>
  <script src="cryptophoto/js/jquery.dataTables.min.js" type="text/javascript"></script>
  <link href="cryptophoto/css/demo_table_jui.css" rel="stylesheet" media="all" type="text/css" />
  </head>
  <body>
  $header
  <div class="container">
    <table width="700" id="reptable" class="display ex_highlight">
      <thead>
        <tr>
          <td align="center" width="420" nowrap="nowrap" style="cursor: pointer"><b>User</b></td>
          <td align="center" width="140" nowrap="nowrap" ><b>Status</b></td>
          <td align="center" width="140" nowrap="nowrap" ><b>Action</b></td>
        </tr>
      </thead>
      <tbody>
      </tbody>
    </table>
  </div>
  <script src="cryptophoto/js/users.js" type="text/javascript"></script>
  </body>
  </html>
HTML

  print "Content-Type: text/html\n\n";
  print $HTML;

}


sub do_index {
  my($FORM, $sc) = @_;

  my $header = header("index", $sc);

  my $HTML =<<HTML;
<!DOCTYPE html>
<html>
  <head>
  <link href="cryptophoto/css/style.css" rel="stylesheet" media="all" type="text/css" />
  </head>
  <body>
    $header
    <ul class="icons">
      <li>
        <a href="$sc/cgi/addon_cryptophoto.cgi?cmd=settings"><img src="cryptophoto/images/service_v2.gif" border="0"></a>
        <br />
        <a href="$sc/cgi/addon_cryptophoto.cgi?cmd=settings">Service<br/>Settings</a>
      </li>
      <li>
        <a href="$sc/cgi/addon_cryptophoto.cgi?cmd=users"><img src="cryptophoto/images/account-info_v2.gif" border="0"></a>
        <br />
        <a href="$sc/cgi/addon_cryptophoto.cgi?cmd=users">Users Management</a>
      </li>
      <li>
        <a href="$sc/securitypolicy_configure"><img src="cryptophoto/images/security-policy_v2.gif" border="0"></a>
        <br />
        <a href="$sc/securitypolicy_configure">Configure Security Policies</a>
      </li>
    </ul>
  </body>
</html>
HTML

  print "Content-Type: text/html\n\n";
  print $HTML;
}

sub do_users_list {
  my($FORM, $sc) = @_;

  my $users = get_users();

  my $info = "";

  foreach my $user (@{$users}) {
    my $active = active($user);
    $info .= '["' . $user . '","<center>' .( $active ? "Active" : "Inactive"). 
             '</center>", "<center><a href=\'javascript:void(0)\' onclick=\'fnUserSwitch(this, \\"'.$user.'\\");return false;\'>'. ( $active ? "Deactivate" : "Activate").
             '</a>"],';
  }
  $info =~ s/,$//;
  
  my $DATA ="{\"aaData\":[$info]}";

  print "Content-Type: application/json\n\n";
  print $DATA;

}

sub do_user_switch {
  my($FORM, $sc) = @_;

  my $user = $FORM->{'id'};

  my $active = active($user);

  if ($active) {
    active($user, 0);
  } else {
    active($user, 1);
  }

  print "Content-Type: application/json\n\n";
  print '{"success":"OK"}';
}


sub header {
  my($cmd, $sc) = @_;

  my $ret = "";

  if ($cmd eq "settings") {
    $ret = "<div id='breadcrumbs'><p>&nbsp;<a href='$sc/scripts/command?PFILE=main'>Main</a>&nbsp;&gt;&gt;&nbsp;<a href='$sc/scripts/command?PFILE=Plugins'>Plugins</a>&nbsp;&gt;&gt;&nbsp;<a href='$sc/cgi/addon_cryptophoto.cgi'>CryptoPhoto</a>&nbsp;&gt;&gt;&nbsp;<b>Settings</b></p></div>";
  }
  elsif ($cmd eq "users") {
    $ret = "<div id='breadcrumbs'><p>&nbsp;<a href='$sc/scripts/command?PFILE=main'>Main</a>&nbsp;&gt;&gt;&nbsp;<a href='$sc/scripts/command?PFILE=Plugins'>Plugins</a>&nbsp;&gt;&gt;&nbsp;<a href='$sc/cgi/addon_cryptophoto.cgi'>CryptoPhoto</a>&nbsp;&gt;&gt;&nbsp;<b>Users Management</b></p></div>";
  }
  elsif ($cmd eq "index") {
    $ret = "<div id='breadcrumbs'><p>&nbsp;<a href='$sc/scripts/command?PFILE=main'>Main</a>&nbsp;&gt;&gt;&nbsp;<a href='$sc/scripts/command?PFILE=Plugins'>Plugins</a>&nbsp;&gt;&gt;&nbsp;<b>CryptoPhoto</b></p></div>";
  }

  return $ret;
}

sub active {
  my ($user, $value) = @_;

  my $settings_file = '/usr/local/cpanel/base/3rdparty/cryptophoto/users/' . $user;

  if (-e $settings_file ) {
    my $conf = Cpanel::CryptoPhoto::Utils::load_Config($settings_file);

    if (defined $value) {
      $conf->{'enabled'} = $value;
      Cpanel::CryptoPhoto::Utils::flushConfig($conf, $settings_file);
    }
    return $conf->{'enabled'};
  }
  return 0;
}

sub get_users {
  my $user_dir = '/usr/local/cpanel/base/3rdparty/cryptophoto/users/';
  my $users = [];
  opendir(D, $user_dir) || return $users;
  while (my $f = readdir(D)) {
    next if($f eq "." || $f eq "..");
    push @{$users}, $f;
  }
  closedir(D);
  return $users;
}
